#include <xf86Parser.h>
#include <glib.h>
#include <string.h>


static int
launch_setxkbmap (const char *xkb_rules,
		  const char *xkb_model,
		  const char *xkb_layout)
{
  char *command;
  char **argv;

  if (xkb_rules == NULL || xkb_model == NULL || xkb_layout == NULL)
    return 1;

  command = g_strdup_printf ("/usr/X11R6/bin/setxkbmap -rules %s -model %s -layout %s",
			     xkb_rules, xkb_model, xkb_layout);
  g_shell_parse_argv (command,
		      NULL, &argv,
		      NULL);
  g_spawn_async ("/", argv, NULL,
		 G_SPAWN_DO_NOT_REAP_CHILD,
		 NULL, NULL, NULL, NULL);
  g_strfreev (argv);
  g_free (command);

  return 0;
}

int
set_xkb_keymap (void)
{
  const char *config_file;
  XF86ConfigPtr config = NULL;
  XF86ConfLayoutPtr layout;
  XF86ConfInputrefPtr inputref;
  XF86ConfInputPtr keyboard = NULL;
  XF86OptionPtr option;
  gchar *xkb_rules = NULL;
  gchar *xkb_model = NULL;
  gchar *xkb_layout = NULL;

  config_file = xf86openConfigFile (NULL, NULL, NULL);
  if (config_file)
    {
      config = xf86readConfigFile ();
      xf86closeConfigFile ();
    }

  if (!config)
    return 1;

  /* We know that XF86Config defaults to the first layout unless explicitly told
   * to use a different one.
   */
  layout = config->conf_layout_lst;
  if (!layout)
    {
      return 1;
    }

  for (inputref = layout->lay_input_lst; inputref; inputref = ((GenericListRec*)inputref)->next)
    {
      for (option = inputref->iref_option_lst; option; option = ((GenericListRec*)option)->next)
	{
	  /* We can only have one CoreKeyboard */
	  if (option->opt_name && !strcmp (option->opt_name, "CoreKeyboard"))
	    {
	      keyboard = inputref->iref_inputdev;
	      break;
	    }
	}
      if (keyboard)
	break;
    }

  if (! keyboard)
    {
      return 1;
    }

  for (option = keyboard->inp_option_lst; option; option = ((GenericListRec*)option)->next)
    {
      if (option->opt_name && option->opt_val && !strcmp (option->opt_name, "XkbRules"))
	{
	  xkb_rules = g_strdup (option->opt_val);
	  continue;
	}
      if (option->opt_name && option->opt_val && !strcmp (option->opt_name, "XkbModel"))
	{
	  xkb_model = g_strdup (option->opt_val);
	  continue;
	}
      if (option->opt_name && option->opt_val && !strcmp (option->opt_name, "XkbLayout"))
	{
	  xkb_layout = g_strdup (option->opt_val);
	  continue;
	}
    }

  
  return launch_setxkbmap (xkb_rules, xkb_model, xkb_layout);
;
}



void
ErrorF(const char *format, ...)
{
  gchar *buffer;
  va_list args;

  va_start (args, format);
  buffer = g_strdup_vprintf (format, args);
  va_end (args);

  g_print ("%s", buffer);
}

void
VErrorF(const char *format, va_list args)
{
  gchar *buffer;

  buffer = g_strdup_vprintf (format, args);

  g_print ("%s", buffer);
}
